from __future__ import annotations

import re
from pathlib import Path
from typing import Any, List, Tuple, Union, Optional

import joblib
import requests
from retrying import retry

from _qwak_proto.qwak.builds.build_url_pb2 import BuildVersioningTagsType
from _qwak_proto.qwak.builds.builds_orchestrator_service_pb2 import (
    GetBuildVersioningUploadURLResponse,
)
from qwak.exceptions import QwakException
from qwak.exceptions import QwakGeneralBuildException
from qwak.inner.build_logic.constants.temp_dir import TEMP_LOCAL_MODEL_DIR
from qwak.inner.build_logic.constants.upload_tag import (
    MODEL_CODE_TAG,
    SKINNY_MODEL_CODE_TAG,
    QWAK_SDK_VERSION_TAG,
    BUILD_CONFIG_TAG,
    QWAK_RUNTIME_WHEEL_TAG,
    QWAK_CORE_WHEEL_TAG,
    QWAK_BUILT_MODEL_TAG,
)
from qwak.inner.build_logic.interface.step_inteface import Step
from qwak.inner.build_logic.tools.files import (
    QWAK_IGNORE_FILE_NAME,
    IGNORED_PATTERNS_FOR_UPLOAD,
    zip_model,
    UploadInChunks,
)
from qwak.inner.build_logic.tools.ignore_files import load_patterns_from_ignore_file

_MAX_FILE_SIZE_BYTES = 10000000


def should_retry(qwak_exception: QwakException):
    # when Got 403 from Jfrog it means that the reposity doesn't exist. It may happen when in the first build in the project
    return str(qwak_exception.message).__contains__("403")


class UploadStep(Step):
    STEP_DESCRIPTION = "Saving Qwak Model"

    def description(self) -> str:
        return self.STEP_DESCRIPTION

    def execute(self) -> None:
        files_tag_iterator = self.create_files_to_upload()
        files_total_size = sum(
            file.stat().st_size for (file, tag) in files_tag_iterator
        )
        upload_so_far = 0
        for file, tag in files_tag_iterator:
            if file.exists():
                pre_signed_url = self.upload_file(
                    file=file,
                    tag=tag,
                    all_files_size_to_upload=files_total_size,
                    read_so_far=upload_so_far,
                )
                upload_so_far += file.stat().st_size

                if tag == MODEL_CODE_TAG:
                    self.context.model_code_remote_url = str(pre_signed_url).split("?")[
                        0
                    ]

    def create_files_to_upload(self) -> Union[List[Tuple[Any, Any], Tuple[Path, Any]]]:
        ignored_patterns = (
            load_patterns_from_ignore_file(
                build_logger=self.build_logger,
                ignore_file_path=self.context.host_temp_local_build_dir
                / TEMP_LOCAL_MODEL_DIR
                / self.config.build_properties.model_uri.main_dir
                / QWAK_IGNORE_FILE_NAME,
            )
            + IGNORED_PATTERNS_FOR_UPLOAD
        )

        # copy 'main' and 'tests' directories
        dirs_to_include = [self.config.build_properties.model_uri.main_dir, "tests"]
        deps_folders = []
        for (
            folder
        ) in self.config.build_properties.model_uri.dependency_required_folders:
            destination_folder = folder
            while destination_folder.startswith(".."):
                destination_folder = re.sub(r"^\.\./", "", destination_folder)
            deps_folders.append(destination_folder)
        if deps_folders:
            self.build_logger.debug(
                f"Adding dependency folders to model code: {deps_folders}"
            )
            dirs_to_include += deps_folders

        self.build_logger.debug("Zipping skinny model code")
        skinny_size_zip_file = zip_model(
            build_dir=self.context.host_temp_local_build_dir,
            dependency_file=self.context.model_relative_dependency_file,
            deps_lock_file=self.context.model_relative_dependency_lock_file,
            dirs_to_include=dirs_to_include,
            zip_name="skinny_size_model_code",
            ignored_patterns=ignored_patterns,
            max_bytes=_MAX_FILE_SIZE_BYTES,
        )

        # Full size model
        self.build_logger.debug("Zipping full model code")
        full_size_zip_file = zip_model(
            build_dir=self.context.host_temp_local_build_dir,
            dependency_file=self.context.model_relative_dependency_file,
            deps_lock_file=self.context.model_relative_dependency_lock_file,
            dirs_to_include=dirs_to_include,
            zip_name="full_size_model_code",
            ignored_patterns=ignored_patterns,
        )

        # Dump config file for upload
        config_file_temp = self.context.host_temp_local_build_dir / "build.conf"
        config_file_temp.write_text(self.config.to_yaml())

        # Dump qwak-sdk version for upload
        qwak_sdk_version_temp = self.context.host_temp_local_build_dir / "VERSION"
        qwak_sdk_version_temp.write_text(self.context.qwak_sdk_version)

        files_tag_iterator = [
            (full_size_zip_file, MODEL_CODE_TAG),
            (skinny_size_zip_file, SKINNY_MODEL_CODE_TAG),
            (qwak_sdk_version_temp, QWAK_SDK_VERSION_TAG),
            (config_file_temp, BUILD_CONFIG_TAG),
        ]

        if self.context.custom_runtime_wheel:
            files_tag_iterator.append(
                (self.context.custom_runtime_wheel, QWAK_RUNTIME_WHEEL_TAG)
            )

        if self.context.custom_core_wheel:
            files_tag_iterator.append(
                (self.context.custom_core_wheel, QWAK_CORE_WHEEL_TAG)
            )

        if self.config.pre_built_model:
            temp_model_file = (
                self.context.host_temp_local_build_dir / QWAK_BUILT_MODEL_TAG
            )
            joblib.dump(self.config.pre_built_model, temp_model_file, compress=3)
            files_tag_iterator.append((temp_model_file, QWAK_BUILT_MODEL_TAG))

        return files_tag_iterator

    def upload_file(
        self, file: Path, tag: str, all_files_size_to_upload: int, read_so_far: int
    ):
        self.build_logger.debug(f"Upload file {file}")

        pre_signed_url_response: GetBuildVersioningUploadURLResponse = (
            self.get_pre_signed_upload_url(
                tag=tag, tag_type=BuildVersioningTagsType.FILE_TAG_TYPE
            )
        )
        self.upload_file_to_remote_storge(
            upload_url=pre_signed_url_response.upload_url,
            file=file,
            all_files_size_to_upload=all_files_size_to_upload,
            read_so_far=read_so_far,
            headers=pre_signed_url_response.headers,
        )

        self.build_logger.debug(f"Upload file {file} completed")

        return pre_signed_url_response.upload_url

    def get_pre_signed_upload_url(
        self, tag: str, tag_type: Optional[BuildVersioningTagsType]
    ) -> GetBuildVersioningUploadURLResponse:
        try:
            self.build_logger.debug(f"Getting pre-signed url for upload - tag {tag}")

            pre_signed_url_response: GetBuildVersioningUploadURLResponse = (
                self.context.client_builds_orchestrator.get_build_versioning_upload_url(
                    build_id=self.context.build_id,
                    model_id=self.context.model_id,
                    tag=tag,
                    tag_type=tag_type,
                )
            )

            self.build_logger.debug("Pre-signed url generated successfully")

            return pre_signed_url_response
        except QwakException as e:
            raise QwakGeneralBuildException(
                message="Unable to get pre-signed url for uploading model",
                src_exception=e,
            )

    def upload_file_to_remote_storge(
        self,
        upload_url: str,
        file: Path,
        all_files_size_to_upload: int,
        read_so_far: int,
        headers: Optional[dict] = None,
    ):
        if not headers:
            headers = {}

        try:
            self.build_logger.debug(f"Upload file {file} to Qwak storage")

            self.send_request(
                upload_url, file, all_files_size_to_upload, read_so_far, headers
            )
            self.build_logger.debug(
                f"File {file} uploaded to Qwak storage successfully"
            )
        except Exception as e:
            raise QwakGeneralBuildException(
                message="Fail uploading model to remote storage.",
                src_exception=e,
            )

    @retry(retry_on_exception=should_retry, wait_fixed=15000, stop_max_delay=60000)
    def send_request(
        self,
        upload_url: str,
        file: Path,
        all_files_size_to_upload: int,
        read_so_far: int,
        headers: Optional[dict],
    ):
        if not headers:
            headers = {}

        # Adding to the current headers the content-type
        headers["content-type"] = "text/plain"

        http_response = requests.put(  # nosec B113
            url=upload_url,
            data=UploadInChunks(
                file=file,
                build_logger=self.build_logger,
                chunk_size_bytes=10,
                all_files_size_to_upload=all_files_size_to_upload,
                read_so_far=read_so_far,
            ),
            headers=headers,
        )

        if http_response.status_code not in [200, 201]:
            raise QwakException(
                f"Status: [{http_response.status_code}], "
                f"reason: [{http_response.reason}]"
            )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/traffic/v1/traffic.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dqwak/traffic/v1/traffic.proto\x12\x0fqwak.traffic.v1\"J\n\x13MultipleHttpTraffic\x12\x33\n\rhttp_traffics\x18\x01 \x03(\x0b\x32\x1c.qwak.traffic.v1.HttpTraffic\"~\n\x0bHttpTraffic\x12\n\n\x02id\x18\x01 \x01(\t\x12\x32\n\x0broute_match\x18\x02 \x01(\x0b\x32\x1b.qwak.traffic.v1.RouteMatchH\x00\x12\r\n\x05order\x18\x03 \x01(\x05\x12\x10\n\x08group_id\x18\x04 \x01(\tB\x0e\n\x0chttp_traffic\"k\n\nRouteMatch\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\'\n\x07matches\x18\x02 \x03(\x0b\x32\x16.qwak.traffic.v1.Match\x12&\n\x06routes\x18\x03 \x03(\x0b\x32\x16.qwak.traffic.v1.Route\"E\n\x05Match\x12\x34\n\x0cheader_match\x18\x01 \x01(\x0b\x32\x1c.qwak.traffic.v1.HeaderMatchH\x00\x42\x06\n\x04rule\"_\n\x0bHeaderMatch\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\tcondition\x18\x02 \x01(\x0b\x32 .qwak.traffic.v1.HeaderCondition\x12\x0e\n\x06invert\x18\x03 \x01(\x08\"\xa4\x01\n\x0fHeaderCondition\x12\x42\n\x10\x62inary_condition\x18\x01 \x01(\x0b\x32&.qwak.traffic.v1.HeaderBinaryConditionH\x00\x12@\n\x0funary_condition\x18\x02 \x01(\x0b\x32%.qwak.traffic.v1.HeaderUnaryConditionH\x00\x42\x0b\n\tcondition\"]\n\x14HeaderUnaryCondition\x12\x34\n\x08operator\x18\x01 \x01(\x0e\x32\".qwak.traffic.v1.UnaryOperatorType\x12\x0f\n\x07operand\x18\x02 \x01(\t\"}\n\x15HeaderBinaryCondition\x12\x35\n\x08operator\x18\x01 \x01(\x0e\x32#.qwak.traffic.v1.BinaryOperatorType\x12\x15\n\rfirst_operand\x18\x02 \x01(\t\x12\x16\n\x0esecond_operand\x18\x03 \x01(\t\"h\n\x05Route\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x0e\n\x06weight\x18\x02 \x01(\r\x12\x0e\n\x06shadow\x18\x03 \x01(\x08\x12\x11\n\tvariation\x18\x04 \x01(\t\x12\x1a\n\x12request_timeout_ms\x18\x05 \x01(\x05*\xc4\x02\n\x11UnaryOperatorType\x12\x1f\n\x1bUNARY_OPERATOR_TYPE_INVALID\x10\x00\x12#\n\x1fUNARY_OPERATOR_TYPE_EXACT_MATCH\x10\x01\x12(\n$UNARY_OPERATOR_TYPE_SAFE_REGEX_MATCH\x10\x02\x12%\n!UNARY_OPERATOR_TYPE_PRESENT_MATCH\x10\x03\x12$\n UNARY_OPERATOR_TYPE_PREFIX_MATCH\x10\x04\x12$\n UNARY_OPERATOR_TYPE_SUFFIX_MATCH\x10\x05\x12&\n\"UNARY_OPERATOR_TYPE_CONTAINS_MATCH\x10\x06\x12$\n UNARY_OPERATOR_TYPE_STRING_MATCH\x10\x07*\\\n\x12\x42inaryOperatorType\x12 \n\x1c\x42INARY_OPERATOR_TYPE_INVALID\x10\x00\x12$\n BINARY_OPERATOR_TYPE_RANGE_MATCH\x10\x01\x42\x17\n\x13\x63om.qwak.traffic.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.traffic.v1.traffic_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.qwak.traffic.v1P\001'
  _globals['_UNARYOPERATORTYPE']._serialized_start=1027
  _globals['_UNARYOPERATORTYPE']._serialized_end=1351
  _globals['_BINARYOPERATORTYPE']._serialized_start=1353
  _globals['_BINARYOPERATORTYPE']._serialized_end=1445
  _globals['_MULTIPLEHTTPTRAFFIC']._serialized_start=50
  _globals['_MULTIPLEHTTPTRAFFIC']._serialized_end=124
  _globals['_HTTPTRAFFIC']._serialized_start=126
  _globals['_HTTPTRAFFIC']._serialized_end=252
  _globals['_ROUTEMATCH']._serialized_start=254
  _globals['_ROUTEMATCH']._serialized_end=361
  _globals['_MATCH']._serialized_start=363
  _globals['_MATCH']._serialized_end=432
  _globals['_HEADERMATCH']._serialized_start=434
  _globals['_HEADERMATCH']._serialized_end=529
  _globals['_HEADERCONDITION']._serialized_start=532
  _globals['_HEADERCONDITION']._serialized_end=696
  _globals['_HEADERUNARYCONDITION']._serialized_start=698
  _globals['_HEADERUNARYCONDITION']._serialized_end=791
  _globals['_HEADERBINARYCONDITION']._serialized_start=793
  _globals['_HEADERBINARYCONDITION']._serialized_end=918
  _globals['_ROUTE']._serialized_start=920
  _globals['_ROUTE']._serialized_end=1024
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/streaming_aggregation.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-qwak/execution/v1/streaming_aggregation.proto\x12\x1fqwak.feature.store.execution.v1\"@\n%RowLevelStreamingAggregationIngestion\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"\xf8\x01\n\'CompactionStreamingAggregationIngestion\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12T\n\x14scheduled_compaction\x18\x02 \x01(\x0b\x32\x34.qwak.feature.store.execution.v1.ScheduledCompactionH\x00\x12N\n\x11manual_compaction\x18\x03 \x01(\x0b\x32\x31.qwak.feature.store.execution.v1.ManualCompactionH\x00\x42\x0e\n\x0ctrigger_type\"\x15\n\x13ScheduledCompaction\"\x12\n\x10ManualCompaction\"@\n%StreamingAggregationBackfillIngestion\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\tB)\n%com.qwak.ai.features.execution.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.execution.v1.streaming_aggregation_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.qwak.ai.features.execution.api.v1P\001'
  _globals['_ROWLEVELSTREAMINGAGGREGATIONINGESTION']._serialized_start=82
  _globals['_ROWLEVELSTREAMINGAGGREGATIONINGESTION']._serialized_end=146
  _globals['_COMPACTIONSTREAMINGAGGREGATIONINGESTION']._serialized_start=149
  _globals['_COMPACTIONSTREAMINGAGGREGATIONINGESTION']._serialized_end=397
  _globals['_SCHEDULEDCOMPACTION']._serialized_start=399
  _globals['_SCHEDULEDCOMPACTION']._serialized_end=420
  _globals['_MANUALCOMPACTION']._serialized_start=422
  _globals['_MANUALCOMPACTION']._serialized_end=440
  _globals['_STREAMINGAGGREGATIONBACKFILLINGESTION']._serialized_start=442
  _globals['_STREAMINGAGGREGATIONBACKFILLINGESTION']._serialized_end=506
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.feature_store.entities.entity_pb2
import qwak.feature_store.features.feature_set_types_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ValueType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ValueTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ValueType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _ValueType.ValueType  # 0
    """An invalid state ValueType, the default if unset"""
    STRING: _ValueType.ValueType  # 1
    """String"""
    INT: _ValueType.ValueType  # 2
    """Integer"""

class ValueType(_ValueType, metaclass=_ValueTypeEnumTypeWrapper): ...

INVALID: ValueType.ValueType  # 0
"""An invalid state ValueType, the default if unset"""
STRING: ValueType.ValueType  # 1
"""String"""
INT: ValueType.ValueType  # 2
"""Integer"""
global___ValueType = ValueType

class FeatureSetDeployment(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ExtraDeploymentConfigurationEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class ExtraEnvConfigurationEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    STREAMING_FEATURE_SET_FIELD_NUMBER: builtins.int
    BATCH_FEATURE_SET_FIELD_NUMBER: builtins.int
    STREAMING_AGGREGATION_FEATURE_SET_FIELD_NUMBER: builtins.int
    EXTRA_DEPLOYMENT_CONFIGURATION_FIELD_NUMBER: builtins.int
    EXTRA_ENV_CONFIGURATION_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    FEATURESET_ID_FIELD_NUMBER: builtins.int
    SECRET_SERVICE_URL_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    QWAK_INTERNAL_PROTOCOL_VERSION_FIELD_NUMBER: builtins.int
    FEATURESET_VERSION_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    """FeatureSet name"""
    @property
    def streaming_feature_set(self) -> qwak.feature_store.features.feature_set_types_pb2.StreamingFeatureSetV1: ...
    @property
    def batch_feature_set(self) -> qwak.feature_store.features.feature_set_types_pb2.BatchFeatureSetV1: ...
    @property
    def streaming_aggregation_feature_set(self) -> qwak.feature_store.features.feature_set_types_pb2.StreamingAggregationFeatureSet: ...
    @property
    def extra_deployment_configuration(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Spark conf passthrough"""
    @property
    def extra_env_configuration(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Config map configuration passthrough"""
    @property
    def entity(self) -> qwak.feature_store.entities.entity_pb2.EntityDefinition:
        """Entity definition of the FeatureSet"""
    featureset_id: builtins.str
    """FeatureSet id"""
    secret_service_url: builtins.str
    """Secret service url of the relevant environment"""
    run_id: builtins.int
    """Run id which represents the job id"""
    qwak_internal_protocol_version: builtins.int
    """Qwak featureset protocol version"""
    featureset_version: builtins.int
    """Featureset version"""
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
        streaming_feature_set: qwak.feature_store.features.feature_set_types_pb2.StreamingFeatureSetV1 | None = ...,
        batch_feature_set: qwak.feature_store.features.feature_set_types_pb2.BatchFeatureSetV1 | None = ...,
        streaming_aggregation_feature_set: qwak.feature_store.features.feature_set_types_pb2.StreamingAggregationFeatureSet | None = ...,
        extra_deployment_configuration: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        extra_env_configuration: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        entity: qwak.feature_store.entities.entity_pb2.EntityDefinition | None = ...,
        featureset_id: builtins.str = ...,
        secret_service_url: builtins.str = ...,
        run_id: builtins.int = ...,
        qwak_internal_protocol_version: builtins.int = ...,
        featureset_version: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "entity", b"entity", "feature_set", b"feature_set", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "entity", b"entity", "extra_deployment_configuration", b"extra_deployment_configuration", "extra_env_configuration", b"extra_env_configuration", "feature_set", b"feature_set", "featureset_id", b"featureset_id", "featureset_name", b"featureset_name", "featureset_version", b"featureset_version", "qwak_internal_protocol_version", b"qwak_internal_protocol_version", "run_id", b"run_id", "secret_service_url", b"secret_service_url", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["feature_set", b"feature_set"]) -> typing_extensions.Literal["streaming_feature_set", "batch_feature_set", "streaming_aggregation_feature_set"] | None: ...

global___FeatureSetDeployment = FeatureSetDeployment

class BatchDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATCH_EXECUTION_DATE_FIELD_NUMBER: builtins.int
    TRY_NUMBER_FIELD_NUMBER: builtins.int
    BATCH_TRIGGER_DATE_FIELD_NUMBER: builtins.int
    @property
    def batch_execution_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Execution date"""
    try_number: builtins.int
    """Try number"""
    @property
    def batch_trigger_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Outside system trigger date"""
    def __init__(
        self,
        *,
        batch_execution_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        try_number: builtins.int = ...,
        batch_trigger_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_execution_date", b"batch_execution_date", "batch_trigger_date", b"batch_trigger_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_execution_date", b"batch_execution_date", "batch_trigger_date", b"batch_trigger_date", "try_number", b"try_number"]) -> None: ...

global___BatchDetails = BatchDetails

class ExecutionIdentifier(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTION_ID_FIELD_NUMBER: builtins.int
    execution_id: builtins.str
    def __init__(
        self,
        *,
        execution_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["execution_id", b"execution_id"]) -> None: ...

global___ExecutionIdentifier = ExecutionIdentifier

class FeatureSetDeploymentSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    FEATURE_SET_TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of feature"""
    id: builtins.str
    """id of the feature"""
    @property
    def metadata(self) -> global___FeatureSetUserMetadata:
        """User generated metadata of the feature set"""
    @property
    def entity(self) -> global___EntityDefinition:
        """Entity specification used by the feature"""
    @property
    def feature_set_type(self) -> global___FeatureSetType:
        """The type of the Entity"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        id: builtins.str = ...,
        metadata: global___FeatureSetUserMetadata | None = ...,
        entity: global___EntityDefinition | None = ...,
        feature_set_type: global___FeatureSetType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["entity", b"entity", "feature_set_type", b"feature_set_type", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["entity", b"entity", "feature_set_type", b"feature_set_type", "id", b"id", "metadata", b"metadata", "name", b"name"]) -> None: ...

global___FeatureSetDeploymentSpec = FeatureSetDeploymentSpec

class FeatureSetUserMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    DISPLAY_NAME_FIELD_NUMBER: builtins.int
    owner: builtins.str
    """Feature set owner"""
    description: builtins.str
    """Human readable description of the feature set"""
    display_name: builtins.str
    """User friendly name of the feature set"""
    def __init__(
        self,
        *,
        owner: builtins.str = ...,
        description: builtins.str = ...,
        display_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "display_name", b"display_name", "owner", b"owner"]) -> None: ...

global___FeatureSetUserMetadata = FeatureSetUserMetadata

class EntityDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITY_ID_FIELD_NUMBER: builtins.int
    ENTITY_SPEC_FIELD_NUMBER: builtins.int
    entity_id: builtins.str
    """Assigned unique id"""
    @property
    def entity_spec(self) -> global___EntitySpec:
        """Specifications of an entity"""
    def __init__(
        self,
        *,
        entity_id: builtins.str = ...,
        entity_spec: global___EntitySpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["entity_spec", b"entity_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["entity_id", b"entity_id", "entity_spec", b"entity_spec"]) -> None: ...

global___EntityDefinition = EntityDefinition

class FeatureSetType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAMING_FEATURE_SET_FIELD_NUMBER: builtins.int
    @property
    def streaming_feature_set(self) -> global___StreamingFeatureSet: ...
    def __init__(
        self,
        *,
        streaming_feature_set: global___StreamingFeatureSet | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["set_type", b"set_type", "streaming_feature_set", b"streaming_feature_set"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["set_type", b"set_type", "streaming_feature_set", b"streaming_feature_set"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["set_type", b"set_type"]) -> typing_extensions.Literal["streaming_feature_set"] | None: ...

global___FeatureSetType = FeatureSetType

class StreamingFeatureSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_FIELD_NUMBER: builtins.int
    @property
    def data_source(self) -> global___StreamingDataSource: ...
    def __init__(
        self,
        *,
        data_source: global___StreamingDataSource | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data_source", b"data_source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source", b"data_source"]) -> None: ...

global___StreamingFeatureSet = StreamingFeatureSet

class KafkaSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONSUMER_TOPIC_FIELD_NUMBER: builtins.int
    CONSUMER_BOOTSTRAP_SERVER_FIELD_NUMBER: builtins.int
    CONSUMER_TIMEOUT_FIELD_NUMBER: builtins.int
    CONSUMER_GROUP_FIELD_NUMBER: builtins.int
    AUTO_OFFSET_TYPE_FIELD_NUMBER: builtins.int
    SECRET_USERNAME_FIELD_NUMBER: builtins.int
    SECRET_PASSWORD_FIELD_NUMBER: builtins.int
    CONSUMER_CONFIGURATION_FIELD_NUMBER: builtins.int
    consumer_topic: builtins.str
    consumer_bootstrap_server: builtins.str
    consumer_timeout: builtins.str
    consumer_group: builtins.str
    auto_offset_type: builtins.str
    secret_username: builtins.str
    secret_password: builtins.str
    consumer_configuration: builtins.str
    def __init__(
        self,
        *,
        consumer_topic: builtins.str = ...,
        consumer_bootstrap_server: builtins.str = ...,
        consumer_timeout: builtins.str = ...,
        consumer_group: builtins.str = ...,
        auto_offset_type: builtins.str = ...,
        secret_username: builtins.str = ...,
        secret_password: builtins.str = ...,
        consumer_configuration: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["auto_offset_type", b"auto_offset_type", "consumer_bootstrap_server", b"consumer_bootstrap_server", "consumer_configuration", b"consumer_configuration", "consumer_group", b"consumer_group", "consumer_timeout", b"consumer_timeout", "consumer_topic", b"consumer_topic", "secret_password", b"secret_password", "secret_username", b"secret_username"]) -> None: ...

global___KafkaSource = KafkaSource

class StreamingDataSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_SOURCE_FIELD_NUMBER: builtins.int
    @property
    def kafka_source(self) -> global___KafkaSource: ...
    def __init__(
        self,
        *,
        kafka_source: global___KafkaSource | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["kafka_source", b"kafka_source", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_source", b"kafka_source", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["kafka_source"] | None: ...

global___StreamingDataSource = StreamingDataSource

class EntitySpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    KEYS_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    VALUE_TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name - should be unique per partner id"""
    @property
    def keys(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """All possible entity keys - on customers data sources"""
    description: builtins.str
    """Description"""
    value_type: global___ValueType.ValueType
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        keys: collections.abc.Iterable[builtins.str] | None = ...,
        description: builtins.str = ...,
        value_type: global___ValueType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "keys", b"keys", "name", b"name", "value_type", b"value_type"]) -> None: ...

global___EntitySpec = EntitySpec



"""Top-level package for qwak-core."""

__author__ = "Qwak.ai"
__version__ = "0.4.391"

from qwak.inner.di_configuration import wire_dependencies
from qwak.model.experiment_tracking import log_metric, log_param
from qwak.model_loggers.artifact_logger import load_file, log_file
from qwak.model_loggers.data_logger import load_data, log_data
from qwak.model_loggers.model_logger import load_model, log_model

from .model.decorators.api import api_decorator as api
from .model.decorators.timer import qwak_timer
from .qwak_client.client import QwakClient

_container = wire_dependencies()



import uuid
from typing import Dict

import grpc
from _qwak_proto.qwak.projects.projects_pb2 import (
    CreateProjectRequest,
    CreateProjectResponse,
    DeleteProjectRequest,
    DeleteProjectResponse,
    GetProjectRequest,
    GetProjectResponse,
    ListProjectsRequest,
    ListProjectsResponse,
    Project,
    ProjectSpec,
)
from _qwak_proto.qwak.projects.projects_pb2_grpc import (
    ProjectsManagementServiceServicer,
)
from google.protobuf.timestamp_pb2 import Timestamp


class ProjectManagerServiceMock(ProjectsManagementServiceServicer):
    def __init__(self):
        self.projects: Dict[str, ProjectSpec] = dict()

    def ListProjects(
        self, request: ListProjectsRequest, context
    ) -> ListProjectsResponse:
        return ListProjectsResponse(
            projects=[project for project in self.projects.values()]
        )

    def CreateProject(
        self, request: CreateProjectRequest, context
    ) -> CreateProjectResponse:
        if request.project_name in self.projects.keys():
            description = f"Project with name {request.project_name} already exists"
            context.set_code(grpc.StatusCode.ALREADY_EXISTS)
            context.set_details(description)
            raise Exception(description)

        now = Timestamp()
        now.GetCurrentTime()

        project_spec = ProjectSpec(
            project_id=str(uuid.uuid4()),
            project_name=request.project_name,
            project_description=request.project_description,
            project_status=ProjectSpec.Status.ACTIVE,
            created_at=now,
            last_modified_at=now,
            models_active=0,
            models_count=0,
        )

        self.projects[request.project_name] = project_spec

        return CreateProjectResponse(project=project_spec)

    def GetProject(self, request: GetProjectRequest, context) -> GetProjectResponse:
        matched_projects = list(
            filter(
                lambda project: request.project_id == project.project_id,
                self.projects.values(),
            )
        )

        if not matched_projects:
            description = f"Project with id {request.project_name} was not found"
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details(description)
            raise Exception(description)

        return GetProjectResponse(project=Project(spec=matched_projects[0]))

    def DeleteProject(
        self, request: DeleteProjectRequest, context
    ) -> DeleteProjectResponse:
        matched_projects = list(
            filter(
                lambda project: request.project_id == project.project_id,
                self.projects.values(),
            )
        )

        if not matched_projects:
            description = f"Project with id {request.project_name} was not found"
            context.set_code(grpc.StatusCode.NOT_FOUND)
            context.set_details(description)
            raise Exception(description)

        del self.projects[matched_projects[0].project_name]

        return DeleteProjectResponse(info="Deleted")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _VectorCollectionStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _VectorCollectionStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VectorCollectionStatus.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    COLLECTION_STATUS_INVALID: _VectorCollectionStatus.ValueType  # 0
    """Invalid collection status"""
    COLLECTION_STATUS_CREATE_REQUESTED: _VectorCollectionStatus.ValueType  # 1
    """Collection create requested"""
    COLLECTION_STATUS_CREATED: _VectorCollectionStatus.ValueType  # 2
    """Collection status is created (exists)"""
    COLLECTION_STATUS_CREATE_FAILED: _VectorCollectionStatus.ValueType  # 3
    """Collection status is created (exists)"""
    COLLECTION_STATUS_DELETE_REQUESTED: _VectorCollectionStatus.ValueType  # 4
    """Collection delete was requested"""
    COLLECTION_STATUS_DELETE_FAILED: _VectorCollectionStatus.ValueType  # 5
    """Collection delete was requested"""
    COLLECTION_STATUS_DELETED: _VectorCollectionStatus.ValueType  # 6
    """Collection was deleted"""

class VectorCollectionStatus(_VectorCollectionStatus, metaclass=_VectorCollectionStatusEnumTypeWrapper): ...

COLLECTION_STATUS_INVALID: VectorCollectionStatus.ValueType  # 0
"""Invalid collection status"""
COLLECTION_STATUS_CREATE_REQUESTED: VectorCollectionStatus.ValueType  # 1
"""Collection create requested"""
COLLECTION_STATUS_CREATED: VectorCollectionStatus.ValueType  # 2
"""Collection status is created (exists)"""
COLLECTION_STATUS_CREATE_FAILED: VectorCollectionStatus.ValueType  # 3
"""Collection status is created (exists)"""
COLLECTION_STATUS_DELETE_REQUESTED: VectorCollectionStatus.ValueType  # 4
"""Collection delete was requested"""
COLLECTION_STATUS_DELETE_FAILED: VectorCollectionStatus.ValueType  # 5
"""Collection delete was requested"""
COLLECTION_STATUS_DELETED: VectorCollectionStatus.ValueType  # 6
"""Collection was deleted"""
global___VectorCollectionStatus = VectorCollectionStatus

class _VectorCollectionMetric:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _VectorCollectionMetricEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VectorCollectionMetric.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    COLLECTION_METRIC_INVALID: _VectorCollectionMetric.ValueType  # 0
    """Collection metric is invalid"""
    COLLECTION_METRIC_L2_SQUARED: _VectorCollectionMetric.ValueType  # 1
    """Collection metric defined L2"""
    COLLECTION_METRIC_COSINE: _VectorCollectionMetric.ValueType  # 2
    """Collection metric defined COSINE"""
    COLLECTION_METRIC_DOT_PRODUCT: _VectorCollectionMetric.ValueType  # 3
    """Collection metric defined DOT PRODUCT"""
    COLLECTION_METRIC_L1: _VectorCollectionMetric.ValueType  # 4
    """Collection metric defined L1"""
    COLLECTION_METRIC_HAMMING: _VectorCollectionMetric.ValueType  # 5
    """Collection metric defined HAMMING"""

class VectorCollectionMetric(_VectorCollectionMetric, metaclass=_VectorCollectionMetricEnumTypeWrapper): ...

COLLECTION_METRIC_INVALID: VectorCollectionMetric.ValueType  # 0
"""Collection metric is invalid"""
COLLECTION_METRIC_L2_SQUARED: VectorCollectionMetric.ValueType  # 1
"""Collection metric defined L2"""
COLLECTION_METRIC_COSINE: VectorCollectionMetric.ValueType  # 2
"""Collection metric defined COSINE"""
COLLECTION_METRIC_DOT_PRODUCT: VectorCollectionMetric.ValueType  # 3
"""Collection metric defined DOT PRODUCT"""
COLLECTION_METRIC_L1: VectorCollectionMetric.ValueType  # 4
"""Collection metric defined L1"""
COLLECTION_METRIC_HAMMING: VectorCollectionMetric.ValueType  # 5
"""Collection metric defined HAMMING"""
global___VectorCollectionMetric = VectorCollectionMetric

class _PropertyType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PropertyTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PropertyType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PROPERTY_TYPE_INVALID: _PropertyType.ValueType  # 0
    PROPERTY_TYPE_STRING: _PropertyType.ValueType  # 1
    PROPERTY_TYPE_INT: _PropertyType.ValueType  # 2
    PROPERTY_TYPE_DOUBLE: _PropertyType.ValueType  # 3
    PROPERTY_TYPE_BOOLEAN: _PropertyType.ValueType  # 4
    PROPERTY_TYPE_TIMESTAMP: _PropertyType.ValueType  # 5

class PropertyType(_PropertyType, metaclass=_PropertyTypeEnumTypeWrapper):
    """DataType of a property in a collection"""

PROPERTY_TYPE_INVALID: PropertyType.ValueType  # 0
PROPERTY_TYPE_STRING: PropertyType.ValueType  # 1
PROPERTY_TYPE_INT: PropertyType.ValueType  # 2
PROPERTY_TYPE_DOUBLE: PropertyType.ValueType  # 3
PROPERTY_TYPE_BOOLEAN: PropertyType.ValueType  # 4
PROPERTY_TYPE_TIMESTAMP: PropertyType.ValueType  # 5
global___PropertyType = PropertyType

class QwakMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATED_AT_FIELD_NUMBER: builtins.int
    CREATED_BY_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_AT_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_BY_FIELD_NUMBER: builtins.int
    @property
    def created_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """When the collection was created"""
    created_by: builtins.str
    """User who created the collection"""
    @property
    def last_modified_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """When the collection was last modified"""
    last_modified_by: builtins.str
    """User who modified the collection"""
    def __init__(
        self,
        *,
        created_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        created_by: builtins.str = ...,
        last_modified_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_modified_by: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["created_at", b"created_at", "last_modified_at", b"last_modified_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["created_at", b"created_at", "created_by", b"created_by", "last_modified_at", b"last_modified_at", "last_modified_by", b"last_modified_by"]) -> None: ...

global___QwakMetadata = QwakMetadata

class VectorCollectionVectorizer(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QWAK_MODEL_NAME_FIELD_NUMBER: builtins.int
    qwak_model_name: builtins.str
    def __init__(
        self,
        *,
        qwak_model_name: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["qwak_model_name", b"qwak_model_name", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["qwak_model_name", b"qwak_model_name", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["qwak_model_name"] | None: ...

global___VectorCollectionVectorizer = VectorCollectionVectorizer

class VectorCollectionSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    VECTORIZER_FIELD_NUMBER: builtins.int
    METRIC_FIELD_NUMBER: builtins.int
    DIMENSION_FIELD_NUMBER: builtins.int
    MULTI_TENANCY_ENABLED_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the collection"""
    description: builtins.str
    """User metadata given to this collection"""
    @property
    def vectorizer(self) -> global___VectorCollectionVectorizer:
        """Vectorizing model"""
    metric: global___VectorCollectionMetric.ValueType
    """The metric defined for this collection"""
    dimension: builtins.int
    """Dimension definition for this collection"""
    multi_tenancy_enabled: builtins.bool
    """Whether multi tenancy is enabled for this collection"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        description: builtins.str = ...,
        vectorizer: global___VectorCollectionVectorizer | None = ...,
        metric: global___VectorCollectionMetric.ValueType = ...,
        dimension: builtins.int = ...,
        multi_tenancy_enabled: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vectorizer", b"vectorizer"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["description", b"description", "dimension", b"dimension", "metric", b"metric", "multi_tenancy_enabled", b"multi_tenancy_enabled", "name", b"name", "vectorizer", b"vectorizer"]) -> None: ...

global___VectorCollectionSpec = VectorCollectionSpec

class VectorCollectionDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    COLLECTION_SPEC_FIELD_NUMBER: builtins.int
    id: builtins.str
    """The unique ID of the given collection"""
    @property
    def collection_spec(self) -> global___VectorCollectionSpec:
        """The spec details of the collection"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        collection_spec: global___VectorCollectionSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["collection_spec", b"collection_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["collection_spec", b"collection_spec", "id", b"id"]) -> None: ...

global___VectorCollectionDefinition = VectorCollectionDefinition

class VectorCollection(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    METADATA_FIELD_NUMBER: builtins.int
    DEFINITION_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    @property
    def metadata(self) -> global___QwakMetadata:
        """QWAK generated metadata of the collection"""
    @property
    def definition(self) -> global___VectorCollectionDefinition:
        """Definition of the collection"""
    status: global___VectorCollectionStatus.ValueType
    """Status of the given collection"""
    def __init__(
        self,
        *,
        metadata: global___QwakMetadata | None = ...,
        definition: global___VectorCollectionDefinition | None = ...,
        status: global___VectorCollectionStatus.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["definition", b"definition", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["definition", b"definition", "metadata", b"metadata", "status", b"status"]) -> None: ...

global___VectorCollection = VectorCollection

class Property(google.protobuf.message.Message):
    """a property in a collection"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DATA_TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """property name"""
    data_type: global___PropertyType.ValueType
    """property type"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        data_type: global___PropertyType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_type", b"data_type", "name", b"name"]) -> None: ...

global___Property = Property

class CollectionMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_VECTORS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    num_vectors: builtins.int
    @property
    def properties(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Property]: ...
    def __init__(
        self,
        *,
        num_vectors: builtins.int = ...,
        properties: collections.abc.Iterable[global___Property] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["num_vectors", b"num_vectors", "properties", b"properties"]) -> None: ...

global___CollectionMetadata = CollectionMetadata



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/admiral/secret/v0/system_secret_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.admiral.secret.v0 import secret_pb2 as qwak_dot_admiral_dot_secret_dot_v0_dot_secret__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2qwak/admiral/secret/v0/system_secret_service.proto\x12\x16qwak.admiral.secret.v0\x1a#qwak/admiral/secret/v0/secret.proto\"d\n\x16SetSystemSecretRequest\x12J\n\x07options\x18\x01 \x01(\x0b\x32\x39.qwak.admiral.secret.v0.SetSystemEnvironmentSecretOptions\"\x19\n\x17SetSystemSecretResponse\"d\n\x19\x44\x65leteSystemSecretRequest\x12G\n\nidentifier\x18\x01 \x01(\x0b\x32\x33.qwak.admiral.secret.v0.EnvironmentSecretIdentifier\"\x1c\n\x1a\x44\x65leteSystemSecretResponse\"a\n\x16GetSystemSecretRequest\x12G\n\nidentifier\x18\x01 \x01(\x0b\x32\x33.qwak.admiral.secret.v0.EnvironmentSecretIdentifier\"d\n\x17GetSystemSecretResponse\x12I\n\x11secret_definition\x18\x01 \x01(\x0b\x32..qwak.admiral.secret.v0.SystemSecretDefinition2\xfa\x02\n\x13SystemSecretService\x12r\n\x0fSetSystemSecret\x12..qwak.admiral.secret.v0.SetSystemSecretRequest\x1a/.qwak.admiral.secret.v0.SetSystemSecretResponse\x12{\n\x12\x44\x65leteSystemSecret\x12\x31.qwak.admiral.secret.v0.DeleteSystemSecretRequest\x1a\x32.qwak.admiral.secret.v0.DeleteSystemSecretResponse\x12r\n\x0fGetSystemSecret\x12..qwak.admiral.secret.v0.GetSystemSecretRequest\x1a/.qwak.admiral.secret.v0.GetSystemSecretResponseB\x8c\x01\n\x1d\x63om.qwak.ai.admiral.secret.v0P\x01Zigithub.com/qwak-ai/qwak-platform/services/core/go/admiral/admiral-api/pb/qwak/admiral/secret/v0;secret_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.admiral.secret.v0.system_secret_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.admiral.secret.v0P\001Zigithub.com/qwak-ai/qwak-platform/services/core/go/admiral/admiral-api/pb/qwak/admiral/secret/v0;secret_v0'
  _globals['_SETSYSTEMSECRETREQUEST']._serialized_start=115
  _globals['_SETSYSTEMSECRETREQUEST']._serialized_end=215
  _globals['_SETSYSTEMSECRETRESPONSE']._serialized_start=217
  _globals['_SETSYSTEMSECRETRESPONSE']._serialized_end=242
  _globals['_DELETESYSTEMSECRETREQUEST']._serialized_start=244
  _globals['_DELETESYSTEMSECRETREQUEST']._serialized_end=344
  _globals['_DELETESYSTEMSECRETRESPONSE']._serialized_start=346
  _globals['_DELETESYSTEMSECRETRESPONSE']._serialized_end=374
  _globals['_GETSYSTEMSECRETREQUEST']._serialized_start=376
  _globals['_GETSYSTEMSECRETREQUEST']._serialized_end=473
  _globals['_GETSYSTEMSECRETRESPONSE']._serialized_start=475
  _globals['_GETSYSTEMSECRETRESPONSE']._serialized_end=575
  _globals['_SYSTEMSECRETSERVICE']._serialized_start=578
  _globals['_SYSTEMSECRETSERVICE']._serialized_end=956
# @@protoc_insertion_point(module_scope)



from typing import List, Optional

import grpc
from _qwak_proto.qwak.vectors.v1.collection.collection_pb2 import (
    VectorCollection,
    VectorCollectionMetric,
    VectorCollectionSpec,
    VectorCollectionVectorizer,
)
from _qwak_proto.qwak.vectors.v1.collection.collection_service_pb2 import (
    CreateCollectionRequest,
    DeleteCollectionByIdRequest,
    DeleteCollectionByNameRequest,
    GetCollectionByIdRequest,
    GetCollectionByNameRequest,
    ListCollectionsRequest,
)
from _qwak_proto.qwak.vectors.v1.collection.collection_service_pb2_grpc import (
    VectorCollectionServiceStub,
)
from dependency_injector.wiring import Provide, inject
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class VectorManagementClient:
    @inject
    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._vector_management_service: VectorCollectionServiceStub = (
            VectorCollectionServiceStub(grpc_channel)
        )

    def create_collection(
        self,
        name: str,
        dimension: int,
        description: str = None,
        metric: VectorCollectionMetric = VectorCollectionMetric.COLLECTION_METRIC_L2_SQUARED,
        vectorizer: Optional[str] = None,
        multi_tenant: bool = False,
    ) -> VectorCollection:
        """
        Create a collection
        """
        try:
            return self._vector_management_service.CreateCollection(
                CreateCollectionRequest(
                    collection_spec=VectorCollectionSpec(
                        name=name,
                        description=description,
                        vectorizer=VectorCollectionVectorizer(
                            qwak_model_name=vectorizer
                        ),
                        metric=metric,
                        dimension=dimension,
                        multi_tenancy_enabled=multi_tenant,
                    )
                )
            ).vector_collection

        except grpc.RpcError as e:
            raise QwakException(f"Failed to create collection: {e.details()}")

    def list_collections(self) -> List[VectorCollection]:
        """
        List all vector collections
        """
        try:
            return self._vector_management_service.ListCollections(
                ListCollectionsRequest()
            ).vector_collections

        except grpc.RpcError as e:
            raise QwakException(f"Failed to list collections: {e.details()}")

    def get_collection_by_name(self, name: str) -> VectorCollection:
        """
        Get vector collection by name
        """
        try:
            return self._vector_management_service.GetCollectionByName(
                GetCollectionByNameRequest(name=name)
            ).vector_collection
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to get collection by name '{name}': {e.details()}"
            )

    def get_collection_by_id(self, id: str) -> VectorCollection:
        """
        Get vector collection by id
        """
        try:
            return self._vector_management_service.GetCollectionById(
                GetCollectionByIdRequest(id=id)
            ).vector_collection
        except grpc.RpcError as e:
            raise QwakException(f"Failed to get collection by id '{id}': {e.details()}")

    def delete_collection_by_id(self, id: str) -> None:
        """
        Delete vector collection by id
        """
        try:
            self._vector_management_service.DeleteCollectionById(
                DeleteCollectionByIdRequest(id=id)
            )
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to delete collection by id '{id}': {e.details()}"
            )

    def delete_collection_by_name(self, name: str) -> None:
        """
        Delete vector collection by id
        """
        try:
            self._vector_management_service.DeleteCollectionByName(
                DeleteCollectionByNameRequest(name=name)
            )
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to delete collection by name '{name}': {e.details()}"
            )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.execution.v1.state.execution_state_pb2
import qwak.execution.v1.state.featureset_state_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GetFeaturesetStateByIdRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_ID_FIELD_NUMBER: builtins.int
    featureset_id: builtins.str
    def __init__(
        self,
        *,
        featureset_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_id", b"featureset_id"]) -> None: ...

global___GetFeaturesetStateByIdRequest = GetFeaturesetStateByIdRequest

class GetFeaturesetStateByIdResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_STATE_FIELD_NUMBER: builtins.int
    @property
    def featureset_state(self) -> qwak.execution.v1.state.featureset_state_pb2.FeaturesetState: ...
    def __init__(
        self,
        *,
        featureset_state: qwak.execution.v1.state.featureset_state_pb2.FeaturesetState | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["featureset_state", b"featureset_state"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_state", b"featureset_state"]) -> None: ...

global___GetFeaturesetStateByIdResponse = GetFeaturesetStateByIdResponse

class GetFeaturesetStateByNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_name", b"featureset_name"]) -> None: ...

global___GetFeaturesetStateByNameRequest = GetFeaturesetStateByNameRequest

class GetFeaturesetStateByNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_STATE_FIELD_NUMBER: builtins.int
    @property
    def featureset_state(self) -> qwak.execution.v1.state.featureset_state_pb2.FeaturesetState: ...
    def __init__(
        self,
        *,
        featureset_state: qwak.execution.v1.state.featureset_state_pb2.FeaturesetState | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["featureset_state", b"featureset_state"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_state", b"featureset_state"]) -> None: ...

global___GetFeaturesetStateByNameResponse = GetFeaturesetStateByNameResponse

class GetMultiFeaturesetStateByIdsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_IDS_FIELD_NUMBER: builtins.int
    @property
    def featureset_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        featureset_ids: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_ids", b"featureset_ids"]) -> None: ...

global___GetMultiFeaturesetStateByIdsRequest = GetMultiFeaturesetStateByIdsRequest

class GetMultiFeaturesetStateByIdsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_STATES_FIELD_NUMBER: builtins.int
    @property
    def featureset_states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.execution.v1.state.featureset_state_pb2.FeaturesetState]: ...
    def __init__(
        self,
        *,
        featureset_states: collections.abc.Iterable[qwak.execution.v1.state.featureset_state_pb2.FeaturesetState] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_states", b"featureset_states"]) -> None: ...

global___GetMultiFeaturesetStateByIdsResponse = GetMultiFeaturesetStateByIdsResponse

class GetMultiFeaturesetStateByNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    @property
    def featureset_name(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        featureset_name: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_name", b"featureset_name"]) -> None: ...

global___GetMultiFeaturesetStateByNameRequest = GetMultiFeaturesetStateByNameRequest

class GetMultiFeaturesetStateByNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_STATE_FIELD_NUMBER: builtins.int
    @property
    def featureset_state(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.execution.v1.state.featureset_state_pb2.FeaturesetState]: ...
    def __init__(
        self,
        *,
        featureset_state: collections.abc.Iterable[qwak.execution.v1.state.featureset_state_pb2.FeaturesetState] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_state", b"featureset_state"]) -> None: ...

global___GetMultiFeaturesetStateByNameResponse = GetMultiFeaturesetStateByNameResponse

class UpdateMultiExecutionStateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTION_STATE_FIELD_NUMBER: builtins.int
    @property
    def execution_state(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.execution.v1.state.execution_state_pb2.ExecutionState]: ...
    def __init__(
        self,
        *,
        execution_state: collections.abc.Iterable[qwak.execution.v1.state.execution_state_pb2.ExecutionState] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["execution_state", b"execution_state"]) -> None: ...

global___UpdateMultiExecutionStateRequest = UpdateMultiExecutionStateRequest

class UpdateMultiExecutionStateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateMultiExecutionStateResponse = UpdateMultiExecutionStateResponse

class DeleteFeaturesetStateByIdRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_ID_FIELD_NUMBER: builtins.int
    featureset_id: builtins.str
    def __init__(
        self,
        *,
        featureset_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_id", b"featureset_id"]) -> None: ...

global___DeleteFeaturesetStateByIdRequest = DeleteFeaturesetStateByIdRequest

class DeleteFeaturesetStateByIdResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_DELETED_RECORDS_FIELD_NUMBER: builtins.int
    num_deleted_records: builtins.int
    """number of deleted records"""
    def __init__(
        self,
        *,
        num_deleted_records: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["num_deleted_records", b"num_deleted_records"]) -> None: ...

global___DeleteFeaturesetStateByIdResponse = DeleteFeaturesetStateByIdResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AtomicLiteral(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STRING_LITERAL_FIELD_NUMBER: builtins.int
    INT_LITERAL_FIELD_NUMBER: builtins.int
    BOOL_LITERAL_FIELD_NUMBER: builtins.int
    DOUBLE_LITERAL_FIELD_NUMBER: builtins.int
    TIMESTAMP_LITERAL_FIELD_NUMBER: builtins.int
    string_literal: builtins.str
    int_literal: builtins.int
    """we treat all ints as 64 - no int/long distinction"""
    bool_literal: builtins.bool
    double_literal: builtins.float
    """not distinguishing double vs float"""
    @property
    def timestamp_literal(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        string_literal: builtins.str = ...,
        int_literal: builtins.int = ...,
        bool_literal: builtins.bool = ...,
        double_literal: builtins.float = ...,
        timestamp_literal: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bool_literal", b"bool_literal", "double_literal", b"double_literal", "int_literal", b"int_literal", "literal_type", b"literal_type", "string_literal", b"string_literal", "timestamp_literal", b"timestamp_literal"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bool_literal", b"bool_literal", "double_literal", b"double_literal", "int_literal", b"int_literal", "literal_type", b"literal_type", "string_literal", b"string_literal", "timestamp_literal", b"timestamp_literal"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["literal_type", b"literal_type"]) -> typing_extensions.Literal["string_literal", "int_literal", "bool_literal", "double_literal", "timestamp_literal"] | None: ...

global___AtomicLiteral = AtomicLiteral

class Filter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OR_FIELD_NUMBER: builtins.int
    AND_FIELD_NUMBER: builtins.int
    EQ_FIELD_NUMBER: builtins.int
    NE_FIELD_NUMBER: builtins.int
    GT_FIELD_NUMBER: builtins.int
    GTE_FIELD_NUMBER: builtins.int
    LT_FIELD_NUMBER: builtins.int
    LTE_FIELD_NUMBER: builtins.int
    LIKE_FIELD_NUMBER: builtins.int
    IS_NULL_FIELD_NUMBER: builtins.int
    IS_NOT_NULL_FIELD_NUMBER: builtins.int
    @property
    def eq(self) -> global___Equal: ...
    @property
    def ne(self) -> global___NotEqual: ...
    @property
    def gt(self) -> global___GreaterThan: ...
    @property
    def gte(self) -> global___GreaterThanEqual: ...
    @property
    def lt(self) -> global___LessThan: ...
    @property
    def lte(self) -> global___LessThanEqual: ...
    @property
    def like(self) -> global___Like: ...
    @property
    def is_null(self) -> global___IsNull: ...
    @property
    def is_not_null(self) -> global___IsNotNull: ...
    def __init__(
        self,
        *,
        eq: global___Equal | None = ...,
        ne: global___NotEqual | None = ...,
        gt: global___GreaterThan | None = ...,
        gte: global___GreaterThanEqual | None = ...,
        lt: global___LessThan | None = ...,
        lte: global___LessThanEqual | None = ...,
        like: global___Like | None = ...,
        is_null: global___IsNull | None = ...,
        is_not_null: global___IsNotNull | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["and", b"and", "eq", b"eq", "filter_case", b"filter_case", "gt", b"gt", "gte", b"gte", "is_not_null", b"is_not_null", "is_null", b"is_null", "like", b"like", "lt", b"lt", "lte", b"lte", "ne", b"ne", "or", b"or"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["and", b"and", "eq", b"eq", "filter_case", b"filter_case", "gt", b"gt", "gte", b"gte", "is_not_null", b"is_not_null", "is_null", b"is_null", "like", b"like", "lt", b"lt", "lte", b"lte", "ne", b"ne", "or", b"or"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["filter_case", b"filter_case"]) -> typing_extensions.Literal["or", "and", "eq", "ne", "gt", "gte", "lt", "lte", "like", "is_null", "is_not_null"] | None: ...

global___Filter = Filter

class Or(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEFT_FIELD_NUMBER: builtins.int
    RIGHT_FIELD_NUMBER: builtins.int
    @property
    def left(self) -> global___Filter: ...
    @property
    def right(self) -> global___Filter: ...
    def __init__(
        self,
        *,
        left: global___Filter | None = ...,
        right: global___Filter | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["left", b"left", "right", b"right"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["left", b"left", "right", b"right"]) -> None: ...

global___Or = Or

class And(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEFT_FIELD_NUMBER: builtins.int
    RIGHT_FIELD_NUMBER: builtins.int
    @property
    def left(self) -> global___Filter: ...
    @property
    def right(self) -> global___Filter: ...
    def __init__(
        self,
        *,
        left: global___Filter | None = ...,
        right: global___Filter | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["left", b"left", "right", b"right"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["left", b"left", "right", b"right"]) -> None: ...

global___And = And

class Equal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___Equal = Equal

class NotEqual(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___NotEqual = NotEqual

class GreaterThan(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___GreaterThan = GreaterThan

class GreaterThanEqual(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___GreaterThanEqual = GreaterThanEqual

class LessThan(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___LessThan = LessThan

class LessThanEqual(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    property: builtins.str
    @property
    def value(self) -> global___AtomicLiteral: ...
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        value: global___AtomicLiteral | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property", "value", b"value"]) -> None: ...

global___LessThanEqual = LessThanEqual

class Like(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    PATTERN_FIELD_NUMBER: builtins.int
    property: builtins.str
    pattern: builtins.str
    def __init__(
        self,
        *,
        property: builtins.str = ...,
        pattern: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pattern", b"pattern", "property", b"property"]) -> None: ...

global___Like = Like

class IsNull(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    property: builtins.str
    def __init__(
        self,
        *,
        property: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property"]) -> None: ...

global___IsNull = IsNull

class IsNotNull(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROPERTY_FIELD_NUMBER: builtins.int
    property: builtins.str
    def __init__(
        self,
        *,
        property: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["property", b"property"]) -> None: ...

global___IsNotNull = IsNotNull



from datetime import timedelta
from functools import lru_cache
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import TYPE_CHECKING, Dict, List, Optional, Union

from _qwak_proto.qwak.build.v1.build_pb2 import DESCRIPTOR
from qwak.clients.analytics.client import AnalyticsEngineClient
from qwak.clients.automation_management.client import AutomationsManagementClient
from qwak.clients.batch_job_management.client import BatchJobManagerClient
from qwak.clients.build_orchestrator.client import BuildFilter, BuildOrchestratorClient
from qwak.clients.data_versioning.client import DataVersioningManagementClient
from qwak.clients.data_versioning.data_tag_filter import DataTagFilter
from qwak.clients.deployment.client import DeploymentManagementClient
from qwak.clients.feature_store import FeatureRegistryClient
from qwak.clients.file_versioning.client import FileVersioningManagementClient
from qwak.clients.file_versioning.file_tag_filter import FileTagFilter
from qwak.clients.instance_template.client import InstanceTemplateManagementClient
from qwak.clients.model_management.client import ModelsManagementClient
from qwak.clients.project.client import ProjectsManagementClient
from qwak.exceptions import QwakException
from qwak.feature_store.feature_sets.batch import BatchFeatureSet
from qwak.inner.build_config.build_config_v1 import BuildConfigV1
from qwak.inner.tool import Auth0ClientBase
from qwak.model.base import QwakModel
from qwak.qwak_client.batch_jobs.execution import Execution
from qwak.qwak_client.batch_jobs.task import Task
from qwak.qwak_client.builds.build import Build
from qwak.qwak_client.builds.filters.metric_filter import MetricFilter
from qwak.qwak_client.builds.filters.parameter_filter import ParameterFilter
from qwak.qwak_client.data_versioning.data_tag import DataTag
from qwak.qwak_client.deployments.deployment import Deployment, EnvironmentAudienceRoute
from qwak.qwak_client.file_versioning.file_tag import FileTag
from qwak.qwak_client.models.model import Model
from qwak.qwak_client.models.model_metadata import ModelMetadata
from qwak.qwak_client.projects.project import Project

if TYPE_CHECKING:
    try:
        import pandas as pd
    except ImportError:
        pass


class QwakClient:
    """
    Comprehensive Qwak client to manage operations performed against the Qwak system.
    Acts as a wrapper and single entry point for the different Qwak clients - BatchClient,
    DeploymentClient, and more.
    """

    @lru_cache(maxsize=1)
    def _get_project_management(self):
        return ProjectsManagementClient()

    @lru_cache(maxsize=1)
    def _get_model_management(self):
        return ModelsManagementClient()

    @lru_cache(maxsize=1)
    def _get_build_orchestrator(self):
        return BuildOrchestratorClient()

    @lru_cache(maxsize=1)
    def _get_batch_job_manager(self):
        return BatchJobManagerClient()

    @lru_cache(maxsize=1)
    def _get_file_versioning_management(self):
        return FileVersioningManagementClient()

    @lru_cache(maxsize=1)
    def _get_data_versioning_management(self):
        return DataVersioningManagementClient()

    @lru_cache(maxsize=1)
    def _get_deployment_management(self):
        return DeploymentManagementClient()

    @lru_cache(maxsize=1)
    def _get_feature_registry_client(self):
        return FeatureRegistryClient()

    @lru_cache(maxsize=1)
    def _get_automations_client(self):
        return AutomationsManagementClient()

    @lru_cache(maxsize=1)
    def _get_analytics_engine(self):
        return AnalyticsEngineClient()

    @lru_cache(maxsize=1)
    def _get_instance_template_client(self):
        return InstanceTemplateManagementClient()

    @lru_cache(maxsize=1)
    def _get_auth_client(self, api_key):
        return Auth0ClientBase(api_key)

    def get_token(self, api_key: Optional[str] = None) -> str:
        """
        Retrieve a token by apy key.
        Use Cases:
        Default - from already configured (qwak configured) - conf file
        Support QWAK_API_KEY environment variable
        Get token for a specified API key
        Args:
            api_key (str): a key for token exchange
        Returns:
            str: a client token
        """
        client = self._get_auth_client(api_key)
        return client.get_token()

    def get_latest_build(
        self, model_id: str, build_status: str = "SUCCESSFUL"
    ) -> Optional[str]:
        """
        Get the latest build by its model ID.
        Optionally gets a build_status, by default filters on 'SUCCESSFUL'

        Args:
            model_id (str): The model ID
            build_status (str): build statuses to filter on. Valid values are 'SUCCESSFUL', 'IN_PROGRESS', 'FAILED'

        Returns:
             str: The build ID of the latest build according to the build status. None if no builds match the filter.

        """
        if (
            build_status
            not in DESCRIPTOR.enum_types_by_name["BuildStatus"].values_by_name
        ):
            raise QwakException(
                f"Invalid build status {build_status}. Valid options are 'SUCCESSFUL', 'IN_PROGRESS', 'FAILED'"
            )

        model = self._get_model_management().get_model(model_id=model_id)
        model_uuid = model.uuid

        builds = self._get_build_orchestrator().list_builds(model_uuid=model_uuid).build
        if not builds:
            return None

        builds = [
            build
            for build in builds
            if build.build_status
            == DESCRIPTOR.enum_types_by_name["BuildStatus"]
            .values_by_name[build_status]
            .number
        ]
        builds.sort(key=lambda r: r.audit.created_at.seconds)

        return builds[-1].buildId if builds else None

    def get_builds_by_tags(
        self,
        model_id: str,
        tags: List[str],
        match_any: bool = True,
        include_extra_tags: bool = True,
    ) -> List[Build]:
        """
        Get builds by its model ID and explicit tags

        Args:
            model_id (str): The model ID
            tags (List[str]): List of tags to filter by
            match_any (bool): Whether matching any tag is enough to return a build (default: True)
            include_extra_tags (bool): Whether builds are allowed to have more tags than the ones specified in the tags list (default: True)

        Returns:
             List[Build]: List of builds that contains the requested tags.

        """
        model = self._get_model_management().get_model(model_id=model_id)
        model_uuid = model.uuid

        builds_proto = (
            self._get_build_orchestrator()
            .list_builds(
                model_uuid=model_uuid,
                build_filter=BuildFilter(
                    tags=tags,
                    require_all_tags=not match_any,
                    include_extra_tags=include_extra_tags,
                ),
            )
            .build
        )

        builds = [Build.from_proto(build) for build in builds_proto]

        return builds if builds else None

    def get_build(self, build_id: str) -> Build:
        """
        Get builds by its build ID

        Args:
            build_id (str): The build ID

        Returns:
             Build: The requested build.

        """
        build_proto = self._get_build_orchestrator().get_build(build_id).build

        build = Build.from_proto(build_proto)

        if build:
            model_proto = self._get_model_management().get_model_by_uuid(
                model_uuid=build_proto.model_uuid
            )
            build.model_id = model_proto.model_id

        return build if build else None

    def list_builds(
        self,
        model_id: str,
        tags: List[str],
        filters: List[Union[MetricFilter, ParameterFilter]],
    ) -> List[Build]:
        """
        List builds by its model ID and explicit filters

        Args:
            model_id (str): The model ID
            tags (List[str]): List of tags to filter by
            filters (List[str]): List of metric and parameter filters

        Returns:
             List[Build]: List of builds that contains the requested filters.

        """
        model = self._get_model_management().get_model(model_id=model_id)
        model_uuid = model.uuid

        metric_filters = []
        parameter_filters = []

        for build_filter in filters:
            if isinstance(build_filter, MetricFilter):
                metric_filters.append(MetricFilter.to_proto(build_filter))
            else:
                parameter_filters.append(ParameterFilter.to_proto(build_filter))

        builds_proto = (
            self._get_build_orchestrator()
            .list_builds(
                model_uuid=model_uuid,
                build_filter=BuildFilter(
                    tags=tags,
                    metric_filters=metric_filters,
                    parameter_filters=parameter_filters,
                ),
            )
            .build
        )

        return [Build.from_proto(build) for build in builds_proto]

    def list_file_tags(
        self,
        model_id: str,
        build_id: str = "",
        filter: Optional[FileTagFilter] = None,
    ) -> List[FileTag]:
        """
        List file tags by its model ID

        Args:
            model_id (str): The model ID
            build_id (str): The build ID - optional.
                If not specified, returns all model file tags.
            filter (FileTagFilter): Filter list by - optional.
                value (str): The filter value.
                type (enum): The filter type - FILE_TAG_FILTER_TYPE_CONTAINS/FILE_TAG_FILTER_TYPE_PREFIX.
                If not specified, returns all model file tags.


        Returns:
             List[FileTag]: List of file tags with their specifications.

        """
        model_file_tags_proto = (
            self._get_file_versioning_management().get_model_file_tags(
                model_id=model_id,
                build_id=build_id,
                file_tag_filter=None if filter is None else filter.to_proto(),
            )
        )

        return [
            FileTag.from_proto(file_tag) for file_tag in model_file_tags_proto.file_tags
        ]

    def list_data_tags(
        self,
        model_id: str,
        build_id: str = "",
        filter: Optional[DataTagFilter] = None,
    ) -> List[DataTag]:
        """
        List data tags by its model ID

        Args:
            model_id (str): The model ID
            build_id (str): The build ID - optional.
                If not specified, returns all model data tags.
            filter (DataTagFilter): Filter list by - optional.
                value (str): The filter value.
                type (enum): The filter type - DATA_TAG_FILTER_TYPE_CONTAINS/DATA_TAG_FILTER_TYPE_PREFIX.
                If not specified, returns all model file tags.

        Returns:
             List[DataTag]: List of data tags with their specifications.

        """
        model_data_tags_proto = (
            self._get_data_versioning_management().get_model_data_tags(
                model_id=model_id,
                build_id=build_id,
                data_tag_filter=None if filter is None else filter.to_proto(),
            )
        )

        return [
            DataTag.from_proto(data_tag) for data_tag in model_data_tags_proto.data_tags
        ]

    def set_tag(self, build_id: str, tag: str) -> None:
        """
        Assign a tag to an existing build

        Args:
            build_id (str): The build ID
            tag (str): The tag to assign

        """
        self._get_build_orchestrator().register_tags(build_id=build_id, tags=[tag])

    def set_tags(self, build_id: str, tags: List[str]) -> None:
        """
        Assign a list of tags to an existing build

        Args:
            build_id (str): The build ID
            tags (List[str]): List of tags to assign
        """
        self._get_build_orchestrator().register_tags(build_id=build_id, tags=tags)

    def create_project(
        self,
        project_name: str,
        project_description: str,
    ) -> str:
        """
        Create project

        Args:
            project_name (str): The requested name
            project_description (str): The requested description

        Returns:
             str: The project ID of the newly created project

        """
        project = self._get_project_management().create_project(
            project_name=project_name,
            project_description=project_description,
        )

        return project.project.project_id

    def get_project(self, project_id: str) -> Optional[Project]:
        """
        Get model by its project ID

        Args:
            project_id (str): The model ID

        Returns:
             Optional[Project]: Project by ID.

        """
        return Project.from_proto(
            self._get_project_management().get_project(project_id).project.spec
        )

    def list_projects(self) -> List[Project]:
        """
        List projects

        Returns:
             List[Project]: List of projects

        """
        projects_proto = self._get_project_management().list_projects().projects

        return [Project.from_proto(project) for project in projects_proto]

    def delete_project(self, project_id: str) -> None:
        """
        Delete project by its project ID

        Args:
            project_id (str): The project ID
        """
        return self._get_project_management().delete_project(project_id=project_id)

    def create_model(
        self,
        project_id: str,
        model_name: str,
        model_description: str,
    ) -> str:
        """
        Create model

        Args:
            project_id (str): The project ID to associate the model
            model_name (str): The requested name
            model_description (str): The requested description

        Returns:
             str: The model ID of the newly created project

        """
        model = self._get_model_management().create_model(
            project_id=project_id,
            model_name=model_name,
            model_description=model_description,
        )

        return model.model_id

    def get_model(self, model_id: str) -> Optional[Model]:
        """
        Get model by its model ID

        Args:
            model_id (str): The model ID

        Returns:
             Optional[Model]: Model by ID.

        """
        return Model.from_proto(
            self._get_model_management().get_model(model_id=model_id)
        )

    def list_model_metadata(self, project_id: str) -> List[ModelMetadata]:
        """
        Lists metadata of all models in a project.
        Args:
            project_id (str): The project ID
        Returns:
            List[ModelMetadata]: List of model metadata
        """
        service_response = self._get_model_management().list_models_metadata(project_id)
        model_metadata_from_service = service_response.model_metadata

        result = []
        for metadata_source in model_metadata_from_service:
            model_metadata = ModelMetadata()

            model_metadata.model = Model.from_proto(metadata_source.model)
            model_metadata.deployed_builds = [
                Build.from_proto(build) for build in metadata_source.build
            ]
            model_metadata.deployments = [
                Deployment.from_proto(deployment)
                for deployment in metadata_source.deployment_details
            ]

            audience_routes = []
            for (
                env_id,
                env,
            ) in metadata_source.audience_routes_grouped_by_environment.items():
                for route in env.audience_routes:
                    audience_routes.append(
                        EnvironmentAudienceRoute.from_proto(route, env_id)
                    )

            model_metadata.audience_routes = audience_routes
            result.append(model_metadata)

        return result

    def list_models(self, project_id: str) -> List[Model]:
        """
        List models

        Args:
            project_id (str): The project ID to list models from

        Returns:
             List[Model]: List of models

        """
        models_proto = (
            self._get_model_management().list_models(project_id=project_id).models
        )

        return [Model.from_proto(model) for model in models_proto]

    def get_model_metadata(self, model_id: str) -> ModelMetadata:
        """
        Get model metadata by its model ID

        Args:
            model_id (str): The model ID

        Returns:
            Model metadata by ID.
        """
        service_response = self._get_model_management().get_model_metadata(model_id)
        model_metadata_from_service = service_response.model_metadata
        model_metadata = ModelMetadata()

        model_metadata.model = Model.from_proto(model_metadata_from_service.model)
        model_metadata.deployed_builds = [
            Build.from_builds_management(build)
            for build in model_metadata_from_service.build
        ]
        model_metadata.deployments = [
            Deployment.from_proto(deployment)
            for deployment in model_metadata_from_service.deployment_details
        ]

        audience_routes = []
        for (
            env_id,
            env,
        ) in model_metadata_from_service.audience_routes_grouped_by_environment.items():
            for route in env.audience_routes:
                audience_routes.append(
                    EnvironmentAudienceRoute.from_proto(env_id, route)
                )
        model_metadata.audience_routes = audience_routes

        return model_metadata

    def list_models_metadata(self, project_id: str) -> List[ModelMetadata]:
        """
        List models metadata

        Args:
            project_id (str): The project ID to list models metadata from

        Returns:
             List[ModelMetadata]: List of models metadata

        """
        models_metadata_proto = (
            self._get_model_management()
            .list_models_metadata(project_id=project_id)
            .model_metadata
        )

        return [
            ModelMetadata.from_proto(model_metadata)
            for model_metadata in models_metadata_proto
        ]

    def delete_model(self, model_id: str) -> None:
        """
        Delete model by its model ID

        Args:
            model_id (str): The model ID
        """
        return self._get_model_management().delete_model(
            project_id=self.get_model(model_id).project_id, model_id=model_id
        )

    def get_deployed_build_id_per_environment(
        self, model_id: str
    ) -> Optional[Dict[str, Dict[str, str]]]:
        """
        Get deployed build ID per environment by its model ID

        Args:
            model_id (str): The model ID

        Returns:
             Dict[str, str]: Map environment to deployed build ID. None if the model is not deployed.

        """
        model = self._get_model_management().get_model(model_id=model_id)
        model_uuid = model.uuid

        deployment_details = self._get_deployment_management().get_deployment_details(
            model_id=model_id, model_uuid=model_uuid
        )

        if not deployment_details:
            return None

        environment_to_deployment_details = (
            deployment_details.environment_to_deployment_details
        )

        return {
            env_id: {
                deployment.variation.name: deployment.build_id
                for deployment in v.deployments_details
            }
            for env_id, v in environment_to_deployment_details.items()
        }

    def run_analytics_query(
        self, query: str, timeout: timedelta = None
    ) -> "pd.DataFrame":
        """
        Runs a Qwak Analytics Query and returns the Pandas DataFrame with the results.

        Args:
            query (str): The query to run
            timeout (timedelta): The timeout for the query - optional.
                If not specified, the function will wait indefinitely.

        Returns:
            pd.DataFrame: The results of the query
        """
        try:
            import pandas as pd
        except ImportError:
            raise QwakException(
                "Missing Pandas dependency required for running an analytics query."
            )
        client = self._get_analytics_engine()
        url = client.get_analytics_data(query=query, timeout=timeout)
        return pd.read_csv(url)

    def list_feature_sets(self) -> List[BatchFeatureSet]:
        """
        List all feature sets

        Returns:
            List[FeatureSet]: List of feature sets
        """
        client = self._get_feature_registry_client()
        feature_set_protos = client.list_feature_sets().feature_families

        return [
            BatchFeatureSet._from_proto(
                feature_set.feature_set_definition.feature_set_spec
            )
            for feature_set in feature_set_protos
        ]

    def delete_feature_set(self, feature_set_name: str):
        """
        Delete a feature set

        Args:
            feature_set_name (str): The feature set name
        """
        client = self._get_feature_registry_client()
        feature_set_def = client.get_feature_set_by_name(
            feature_set_name=feature_set_name
        )
        if feature_set_def:
            client.delete_feature_set(
                feature_set_def.feature_set.feature_set_definition.feature_set_id
            )
        else:
            raise QwakException(f"Feature set named '{feature_set_name}' not found")

    def delete_featureset_version(self, featureset_name: str, version_number: int):
        """
        Delete a feature set version

        Args:
            featureset_name (str): The feature set name
            version_number (int): version number
        """
        client = self._get_feature_registry_client()

        client.delete_featureset_version(
            featureset_name=featureset_name, version_number=version_number
        )

    def set_featureset_active_version(self, featureset_name: str, version_number: int):
        """
        Set active feature set version

        Args:
            featureset_name (str): The feature set name
            version_number (int): version number
        """
        client = self._get_feature_registry_client()

        client.set_active_featureset_version(
            featureset_name=featureset_name, version_number=version_number
        )

    def delete_data_source(self, data_source_name: str):
        """
        Delete a data source

        Args:
            data_source_name (str): The data source name
        """
        client = self._get_feature_registry_client()
        data_source_def = client.get_data_source_by_name(
            data_source_name=data_source_name
        )
        if data_source_def:
            client.delete_data_source(
                data_source_def.data_source.data_source_definition.data_source_id
            )
        else:
            raise QwakException(f"Data source named '{data_source_name}' not found")

    def delete_entity(self, entity_name: str):
        """
        Delete an entity_name

        Args:
            entity_name (str): The entity name
        """
        client = self._get_feature_registry_client()
        entity_def = client.get_entity_by_name(entity_name=entity_name)
        if entity_def:
            client.delete_entity(entity_def.entity.entity_definition.entity_id)
        else:
            raise QwakException(f"Entity named '{entity_name}' not found")

    def trigger_batch_feature_set(self, feature_set_name):
        """
        Triggers a batch feature set ingestion job immediately.

        Args:
            feature_set_name (str): The feature set name
        """
        client = self._get_feature_registry_client()
        client.run_feature_set(feature_set_name=feature_set_name)

    def trigger_automation(self, automation_name):
        """
        Triggers an automation immediately.

        Args:
            automation_name (str): The automation name
        """
        client = self._get_automations_client()
        automation = client.get_automation_by_name(automation_name=automation_name)
        if not automation:
            raise QwakException(
                f"Could not find automation with given name '{automation_name}'"
            )
        client.run_automation(automation_id=automation.id)

    def build_model(
        self,
        model_id: str,
        main_module_path: str = ".",
        dependencies_file: Optional[str] = None,
        dependencies_list: Optional[List[str]] = None,
        tags: Optional[List[str]] = None,
        instance: str = "small",
        gpu_compatible: bool = False,
        run_tests: bool = True,
        validate_build_artifact: bool = True,
        validate_build_artifact_timeout: Optional[int] = None,
        prebuilt_qwak_model: Optional[QwakModel] = None,
        base_image: Optional[str] = None,
        build_name: str = "",
    ):
        from qwak.qwak_client.build_api_helpers.trigger_build_api import (
            trigger_build_api,
        )

        config = BuildConfigV1()
        config.build_properties.model_id = model_id
        config.build_properties.build_name = build_name
        config.build_properties.model_uri.uri = main_module_path
        config.build_env.python_env.dependency_file_path = dependencies_file
        config.build_properties.tags = tags if tags else []
        config.build_env.remote.resources.instance = instance
        config.build_properties.gpu_compatible = gpu_compatible
        config.step.validate_build_artifact = validate_build_artifact
        config.step.tests = run_tests
        config.fetch_base_docker_image_name(
            self._get_build_orchestrator(), self._get_instance_template_client()
        )
        config.pre_built_model = prebuilt_qwak_model
        if validate_build_artifact_timeout:
            config.step.validate_build_artifact_timeout = (
                validate_build_artifact_timeout
            )
        if base_image:
            config.build_env.docker.base_image = base_image
        if dependencies_list:
            with TemporaryDirectory() as temporary_directory:
                requirements_filename = Path(temporary_directory) / "requirements.txt"
                with open(requirements_filename, "w") as temp_dependencies_file:
                    temp_dependencies_file.write("\n".join(dependencies_list))
                config.build_env.python_env.dependency_file_path = requirements_filename
                return trigger_build_api(config)
        else:
            return trigger_build_api(config)

    def list_executions(
        self,
        model_id: str,
        build_id: str = "",
    ) -> List[Execution]:
        """
        List batch executions by its model ID

        Args:
            model_id (str): The model ID
            build_id (str): The build ID - optional.
                If not specified, returns all model executions.


        Returns:
             List[Execution]: List of executions with their specifications.

        """
        model_executions_proto = self._get_batch_job_manager().list_batch_jobs(
            model_id=model_id,
            build_id=build_id,
        )

        return [
            Execution.from_proto(execution)
            for execution in model_executions_proto.batch_jobs
        ]

    def list_execution_tasks(
        self,
        execution_id: str,
    ) -> List[Task]:
        """
        List batch executions tasks by its job ID

        Args:
            execution_id (str): The execution ID


        Returns:
             List[Task]: List of execution tasks with their specifications.

        """
        model_execution_tasks_proto = (
            self._get_batch_job_manager().get_batch_job_details(
                job_id=execution_id,
            )
        )

        return [
            Task.from_proto(task)
            for task in model_execution_tasks_proto.batch_job.task_executions
        ]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _MemoryUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MemoryUnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MemoryUnit.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN_MEMORY_UNIT: _MemoryUnit.ValueType  # 0
    MIB: _MemoryUnit.ValueType  # 1
    GIB: _MemoryUnit.ValueType  # 2

class MemoryUnit(_MemoryUnit, metaclass=_MemoryUnitEnumTypeWrapper): ...

UNKNOWN_MEMORY_UNIT: MemoryUnit.ValueType  # 0
MIB: MemoryUnit.ValueType  # 1
GIB: MemoryUnit.ValueType  # 2
global___MemoryUnit = MemoryUnit

class _GpuType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _GpuTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_GpuType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _GpuType.ValueType  # 0
    NVIDIA_K80: _GpuType.ValueType  # 1
    NVIDIA_V100: _GpuType.ValueType  # 2
    NVIDIA_A100: _GpuType.ValueType  # 3
    NVIDIA_T4: _GpuType.ValueType  # 4
    NVIDIA_A10G: _GpuType.ValueType  # 5
    NVIDIA_L4: _GpuType.ValueType  # 6
    NVIDIA_T4_1_4_15: _GpuType.ValueType  # 7
    """More specific node types
    t4.xl
    """
    NVIDIA_T4_1_8_30: _GpuType.ValueType  # 8
    """t4.2xl"""
    NVIDIA_T4_1_16_60: _GpuType.ValueType  # 9
    """t4.4xl"""
    NVIDIA_A100_80GB_8_96_1360: _GpuType.ValueType  # 10
    """a100.8xl"""
    NVIDIA_V100_1_8_52: _GpuType.ValueType  # 11
    """a100.xl"""
    NVIDIA_V100_4_32_208: _GpuType.ValueType  # 12
    """v100.4xl"""
    NVIDIA_H100_NVL: _GpuType.ValueType  # 13
    """H100 NVL XLarge"""
    NVIDIA_A10: _GpuType.ValueType  # 14
    """A10 Small"""
    NVIDIA_H100_SXM: _GpuType.ValueType  # 15
    """H100 SXM 8XLarge"""
    NVIDIA_H100_40GB: _GpuType.ValueType  # 16
    """H100 40GB 8XLarge"""
    NVIDIA_H100_80GB: _GpuType.ValueType  # 17
    """H100 80GB 8XLarge"""
    NVIDIA_M60: _GpuType.ValueType  # 18
    """M60 XLarge"""

class GpuType(_GpuType, metaclass=_GpuTypeEnumTypeWrapper): ...

INVALID: GpuType.ValueType  # 0
NVIDIA_K80: GpuType.ValueType  # 1
NVIDIA_V100: GpuType.ValueType  # 2
NVIDIA_A100: GpuType.ValueType  # 3
NVIDIA_T4: GpuType.ValueType  # 4
NVIDIA_A10G: GpuType.ValueType  # 5
NVIDIA_L4: GpuType.ValueType  # 6
NVIDIA_T4_1_4_15: GpuType.ValueType  # 7
"""More specific node types
t4.xl
"""
NVIDIA_T4_1_8_30: GpuType.ValueType  # 8
"""t4.2xl"""
NVIDIA_T4_1_16_60: GpuType.ValueType  # 9
"""t4.4xl"""
NVIDIA_A100_80GB_8_96_1360: GpuType.ValueType  # 10
"""a100.8xl"""
NVIDIA_V100_1_8_52: GpuType.ValueType  # 11
"""a100.xl"""
NVIDIA_V100_4_32_208: GpuType.ValueType  # 12
"""v100.4xl"""
NVIDIA_H100_NVL: GpuType.ValueType  # 13
"""H100 NVL XLarge"""
NVIDIA_A10: GpuType.ValueType  # 14
"""A10 Small"""
NVIDIA_H100_SXM: GpuType.ValueType  # 15
"""H100 SXM 8XLarge"""
NVIDIA_H100_40GB: GpuType.ValueType  # 16
"""H100 40GB 8XLarge"""
NVIDIA_H100_80GB: GpuType.ValueType  # 17
"""H100 80GB 8XLarge"""
NVIDIA_M60: GpuType.ValueType  # 18
"""M60 XLarge"""
global___GpuType = GpuType

class BuildProperties(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_ID_FIELD_NUMBER: builtins.int
    BRANCH_FIELD_NUMBER: builtins.int
    MODEL_ID_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    MODEL_URI_FIELD_NUMBER: builtins.int
    GPU_COMPATIBLE_FIELD_NUMBER: builtins.int
    build_id: builtins.str
    """Build id"""
    branch: builtins.str
    """Branch name. When none provided - 'main' will be the selected branch"""
    model_id: builtins.str
    """Model id"""
    @property
    def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Build tags"""
    @property
    def model_uri(self) -> global___ModelUriSpec:
        """Model URI spec"""
    gpu_compatible: builtins.bool
    """Is GPU Compatible"""
    def __init__(
        self,
        *,
        build_id: builtins.str = ...,
        branch: builtins.str = ...,
        model_id: builtins.str = ...,
        tags: collections.abc.Iterable[builtins.str] | None = ...,
        model_uri: global___ModelUriSpec | None = ...,
        gpu_compatible: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model_uri", b"model_uri"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["branch", b"branch", "build_id", b"build_id", "gpu_compatible", b"gpu_compatible", "model_id", b"model_id", "model_uri", b"model_uri", "tags", b"tags"]) -> None: ...

global___BuildProperties = BuildProperties

class ModelUriSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    URI_FIELD_NUMBER: builtins.int
    GIT_BRANCH_FIELD_NUMBER: builtins.int
    GIT_CREDENTIALS_FIELD_NUMBER: builtins.int
    GIT_CREDENTIALS_SECRET_FIELD_NUMBER: builtins.int
    COMMIT_ID_FIELD_NUMBER: builtins.int
    MAIN_DIR_FIELD_NUMBER: builtins.int
    uri: builtins.str
    """Model code to build upon, can be a download link or a git repo"""
    git_branch: builtins.str
    """Model code branch to clone"""
    git_credentials: builtins.str
    """Git credentials"""
    git_credentials_secret: builtins.str
    """Secret to fetch git credentials, overrides git_credentials if set"""
    commit_id: builtins.str
    """Commit id - filled when using local model code which is part of a git repo"""
    main_dir: builtins.str
    """Overrides the models main dir"""
    def __init__(
        self,
        *,
        uri: builtins.str = ...,
        git_branch: builtins.str = ...,
        git_credentials: builtins.str = ...,
        git_credentials_secret: builtins.str = ...,
        commit_id: builtins.str = ...,
        main_dir: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit_id", b"commit_id", "git_branch", b"git_branch", "git_credentials", b"git_credentials", "git_credentials_secret", b"git_credentials_secret", "main_dir", b"main_dir", "uri", b"uri"]) -> None: ...

global___ModelUriSpec = ModelUriSpec

class BuildEnv(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOCKER_ENV_FIELD_NUMBER: builtins.int
    PYTHON_ENV_FIELD_NUMBER: builtins.int
    @property
    def docker_env(self) -> global___DockerEnv: ...
    @property
    def python_env(self) -> global___PythonEnv: ...
    def __init__(
        self,
        *,
        docker_env: global___DockerEnv | None = ...,
        python_env: global___PythonEnv | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["docker_env", b"docker_env", "python_env", b"python_env"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["docker_env", b"docker_env", "python_env", b"python_env"]) -> None: ...

global___BuildEnv = BuildEnv

class DockerEnv(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BASE_IMAGE_FIELD_NUMBER: builtins.int
    ENV_VARS_FIELD_NUMBER: builtins.int
    NO_CACHE_FIELD_NUMBER: builtins.int
    ASSUMED_IAM_ROLE_ARN_FIELD_NUMBER: builtins.int
    SERVICE_ACCOUNT_KEY_SECRET_NAME_FIELD_NUMBER: builtins.int
    base_image: builtins.str
    """Base image to use for serving"""
    @property
    def env_vars(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Environment variables for the model code"""
    no_cache: builtins.bool
    """Don't use remote cache to build the images, defaults to use cache,
    if the model dependencies changed significantly it could be faster and better to run with no cache
    """
    assumed_iam_role_arn: builtins.str
    """Custom IAM role to be assumed by model"""
    service_account_key_secret_name: builtins.str
    """Service account key secret name for gcp"""
    def __init__(
        self,
        *,
        base_image: builtins.str = ...,
        env_vars: collections.abc.Iterable[builtins.str] | None = ...,
        no_cache: builtins.bool = ...,
        assumed_iam_role_arn: builtins.str = ...,
        service_account_key_secret_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["assumed_iam_role_arn", b"assumed_iam_role_arn", "base_image", b"base_image", "env_vars", b"env_vars", "no_cache", b"no_cache", "service_account_key_secret_name", b"service_account_key_secret_name"]) -> None: ...

global___DockerEnv = DockerEnv

class PythonEnv(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GIT_CREDENTIALS_FIELD_NUMBER: builtins.int
    GIT_CREDENTIALS_SECRET_FIELD_NUMBER: builtins.int
    QWAK_SDK_VERSION_FIELD_NUMBER: builtins.int
    QWAK_SDK_EXTRA_INDEX_FIELD_NUMBER: builtins.int
    git_credentials: builtins.str
    """Git credentials"""
    git_credentials_secret: builtins.str
    """Secret to fetch git credentials, overrides git_credentials if set"""
    qwak_sdk_version: builtins.str
    """Qwak sdk version of calling user"""
    qwak_sdk_extra_index: builtins.str
    """Pip extra index url to install qwak-sdk from"""
    def __init__(
        self,
        *,
        git_credentials: builtins.str = ...,
        git_credentials_secret: builtins.str = ...,
        qwak_sdk_version: builtins.str = ...,
        qwak_sdk_extra_index: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["git_credentials", b"git_credentials", "git_credentials_secret", b"git_credentials_secret", "qwak_sdk_extra_index", b"qwak_sdk_extra_index", "qwak_sdk_version", b"qwak_sdk_version"]) -> None: ...

global___PythonEnv = PythonEnv

class CpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    cpu: builtins.float
    """Amount of CPU cores"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnit.ValueType
    """Units type of memory"""
    def __init__(
        self,
        *,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu", b"cpu", "memory_amount", b"memory_amount", "memory_units", b"memory_units"]) -> None: ...

global___CpuResources = CpuResources

class GpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GPU_TYPE_FIELD_NUMBER: builtins.int
    GPU_AMOUNT_FIELD_NUMBER: builtins.int
    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    gpu_type: global___GpuType.ValueType
    """The type of the GPU"""
    gpu_amount: builtins.int
    """Amount of GPUs"""
    cpu: builtins.float
    """Amount of CPU cores"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnit.ValueType
    """Units type of memory"""
    def __init__(
        self,
        *,
        gpu_type: global___GpuType.ValueType = ...,
        gpu_amount: builtins.int = ...,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu", b"cpu", "gpu_amount", b"gpu_amount", "gpu_type", b"gpu_type", "memory_amount", b"memory_amount", "memory_units", b"memory_units"]) -> None: ...

global___GpuResources = GpuResources



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ActualValuesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    ACTUALS_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """List of feature columns to fetch."""
    @property
    def entity(self) -> global___Entity:
        """Feature is requested for a single entity (user id for example)."""
    @property
    def actuals(self) -> global___Actuals: ...
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        entity: global___Entity | None = ...,
        actuals: global___Actuals | None = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["actuals", b"actuals", "entity", b"entity", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actuals", b"actuals", "entity", b"entity", "model_id", b"model_id", "timestamp", b"timestamp"]) -> None: ...

global___ActualValuesRequest = ActualValuesRequest

class EnrichedActualValuesData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    ACTUALS_FIELD_NUMBER: builtins.int
    QWAK_ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """List of feature columns to fetch."""
    @property
    def entity(self) -> global___Entity:
        """Feature is requested for a single entity (user id for example)."""
    @property
    def actuals(self) -> global___Actuals: ...
    qwak_environment_id: builtins.str
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        entity: global___Entity | None = ...,
        actuals: global___Actuals | None = ...,
        qwak_environment_id: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["actuals", b"actuals", "entity", b"entity", "timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["actuals", b"actuals", "entity", b"entity", "model_id", b"model_id", "qwak_environment_id", b"qwak_environment_id", "timestamp", b"timestamp"]) -> None: ...

global___EnrichedActualValuesData = EnrichedActualValuesData

class Entity(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Entity name - such as uuid, site id, etc."""
    value: builtins.str
    """Entity value to fetch"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "value", b"value"]) -> None: ...

global___Entity = Entity

class Actuals(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    tag: builtins.str
    """Actual tag - such as label name, etc."""
    @property
    def value(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Entity value to fetch"""
    def __init__(
        self,
        *,
        tag: builtins.str = ...,
        value: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tag", b"tag", "value", b"value"]) -> None: ...

global___Actuals = Actuals

class ActualValuesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ValueStatus:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ValueStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActualValuesResponse._ValueStatus.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID: ActualValuesResponse._ValueStatus.ValueType  # 0
        FAILURE: ActualValuesResponse._ValueStatus.ValueType  # 1
        """Feature value could not be obtained"""
        SUCCESS: ActualValuesResponse._ValueStatus.ValueType  # 2
        """Feature value is set"""

    class ValueStatus(_ValueStatus, metaclass=_ValueStatusEnumTypeWrapper): ...
    INVALID: ActualValuesResponse.ValueStatus.ValueType  # 0
    FAILURE: ActualValuesResponse.ValueStatus.ValueType  # 1
    """Feature value could not be obtained"""
    SUCCESS: ActualValuesResponse.ValueStatus.ValueType  # 2
    """Feature value is set"""

    STATUS_FIELD_NUMBER: builtins.int
    status: global___ActualValuesResponse.ValueStatus.ValueType
    """Feature value status"""
    def __init__(
        self,
        *,
        status: global___ActualValuesResponse.ValueStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["status", b"status"]) -> None: ...

global___ActualValuesResponse = ActualValuesResponse

class ConfigureFeedbackRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ModelType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ModelTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ConfigureFeedbackRequest._ModelType.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID_MODEL_TYPE: ConfigureFeedbackRequest._ModelType.ValueType  # 0
        BINARY_CLASSIFICATION: ConfigureFeedbackRequest._ModelType.ValueType  # 1
        MULTICLASS_CLASSIFICATION: ConfigureFeedbackRequest._ModelType.ValueType  # 2
        REGRESSION: ConfigureFeedbackRequest._ModelType.ValueType  # 3

    class ModelType(_ModelType, metaclass=_ModelTypeEnumTypeWrapper): ...
    INVALID_MODEL_TYPE: ConfigureFeedbackRequest.ModelType.ValueType  # 0
    BINARY_CLASSIFICATION: ConfigureFeedbackRequest.ModelType.ValueType  # 1
    MULTICLASS_CLASSIFICATION: ConfigureFeedbackRequest.ModelType.ValueType  # 2
    REGRESSION: ConfigureFeedbackRequest.ModelType.ValueType  # 3

    MODEL_ID_FIELD_NUMBER: builtins.int
    ANALYTICS_ENTITY_COLUMN_FIELD_NUMBER: builtins.int
    FEEDBACK_ENTITY_COLUMN_FIELD_NUMBER: builtins.int
    FEEDBACK_CONFIG_FIELD_NUMBER: builtins.int
    MODEL_TYPE_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """Feedback model_id"""
    analytics_entity_column: builtins.str
    """Analytics entity column"""
    feedback_entity_column: builtins.str
    """Feedback entity column"""
    @property
    def feedback_config(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FeedbackConfig]:
        """Analytics data config"""
    model_type: global___ConfigureFeedbackRequest.ModelType.ValueType
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
        analytics_entity_column: builtins.str = ...,
        feedback_entity_column: builtins.str = ...,
        feedback_config: collections.abc.Iterable[global___FeedbackConfig] | None = ...,
        model_type: global___ConfigureFeedbackRequest.ModelType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["analytics_entity_column", b"analytics_entity_column", "feedback_config", b"feedback_config", "feedback_entity_column", b"feedback_entity_column", "model_id", b"model_id", "model_type", b"model_type"]) -> None: ...

global___ConfigureFeedbackRequest = ConfigureFeedbackRequest

class FeedbackConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OUTPUT_COLUMN_FIELD_NUMBER: builtins.int
    ACTUAL_TAG_FIELD_NUMBER: builtins.int
    output_column: builtins.str
    actual_tag: builtins.str
    def __init__(
        self,
        *,
        output_column: builtins.str = ...,
        actual_tag: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["actual_tag", b"actual_tag", "output_column", b"output_column"]) -> None: ...

global___FeedbackConfig = FeedbackConfig

class ConfigureFeedbackResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ValueStatus:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ValueStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ConfigureFeedbackResponse._ValueStatus.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID: ConfigureFeedbackResponse._ValueStatus.ValueType  # 0
        FAILURE: ConfigureFeedbackResponse._ValueStatus.ValueType  # 1
        """Feedback configuration could not be obtained"""
        SUCCESS: ConfigureFeedbackResponse._ValueStatus.ValueType  # 2
        """Feedback configuration is set"""

    class ValueStatus(_ValueStatus, metaclass=_ValueStatusEnumTypeWrapper): ...
    INVALID: ConfigureFeedbackResponse.ValueStatus.ValueType  # 0
    FAILURE: ConfigureFeedbackResponse.ValueStatus.ValueType  # 1
    """Feedback configuration could not be obtained"""
    SUCCESS: ConfigureFeedbackResponse.ValueStatus.ValueType  # 2
    """Feedback configuration is set"""

    STATUS_FIELD_NUMBER: builtins.int
    status: global___ConfigureFeedbackResponse.ValueStatus.ValueType
    """Config value status"""
    def __init__(
        self,
        *,
        status: global___ConfigureFeedbackResponse.ValueStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["status", b"status"]) -> None: ...

global___ConfigureFeedbackResponse = ConfigureFeedbackResponse
